@@ -46,6 +46,7 @@
 #include "dstrings.h"
 
 #include "am_map.h"
+extern boolean inhelpscreens; // [crispy]
 
 
 // For use if I do walls with outsides/insides
@@ -101,6 +102,9 @@
 // how much zoom-out per tic
 // pulls out to 0.5x in 1 second
 #define M_ZOOMOUT       ((int) (FRACUNIT/1.02))
+// [crispy] zoom faster with the mouse wheel
+#define M2_ZOOMIN       ((int) (1.08*FRACUNIT))
+#define M2_ZOOMOUT      ((int) (FRACUNIT/1.08))
 
 // translates between frame-buffer and map distances
 // [crispy] fix int overflow that causes map and grid lines to disappear
@@ -460,6 +464,13 @@ void AM_changeWindowLoc(void)
 
     m_x2 = m_x + m_w;
     m_y2 = m_y + m_h;
+
+    // [crispy] reset after moving with the mouse
+    if (f_oldloc.y == INT_MAX)
+    {
+	m_paninc.x = 0;
+	m_paninc.y = 0;
+    }
 }
 
 
@@ -694,6 +705,32 @@ AM_Responder
 	    rc = true;
 	}
     }
+    // [crispy] zoom and move Automap with the mouse (wheel)
+    else if (ev->type == ev_mouse && !crispy->automapoverlay && !menuactive && !inhelpscreens)
+    {
+	if (mousebprevweapon >= 0 && ev->data1 & (1 << mousebprevweapon))
+	{
+		mtof_zoommul = M2_ZOOMOUT;
+		ftom_zoommul = M2_ZOOMIN;
+		rc = true;
+	}
+	else
+	if (mousebnextweapon >= 0 && ev->data1 & (1 << mousebnextweapon))
+	{
+		mtof_zoommul = M2_ZOOMIN;
+		ftom_zoommul = M2_ZOOMOUT;
+		rc = true;
+	}
+	else
+	if (!followplayer && (ev->data2 || ev->data3))
+	{
+		// [crispy] mouse sensitivity for strafe
+		m_paninc.x = FTOM(ev->data2*(mouseSensitivity_x2+5)/80);
+		m_paninc.y = FTOM(ev->data3*(mouseSensitivity_x2+5)/80);
+		f_oldloc.y = INT_MAX;
+		rc = true;
+	}
+    }
     else if (ev->type == ev_keydown)
     {
 	rc = true;
@@ -779,7 +816,6 @@ AM_Responder
         else if (key == key_map_overlay)
         {
             // [crispy] force redraw status bar
-            extern boolean inhelpscreens;
             inhelpscreens = true;
 
             crispy->automapoverlay = !crispy->automapoverlay;
@@ -851,6 +887,13 @@ void AM_changeWindowScale(void)
     scale_mtof = FixedMul(scale_mtof, mtof_zoommul);
     scale_ftom = FixedDiv(FRACUNIT, scale_mtof);
 
+    // [crispy] reset after zooming with the mouse wheel
+    if (ftom_zoommul == M2_ZOOMIN || ftom_zoommul == M2_ZOOMOUT)
+    {
+	mtof_zoommul = FRACUNIT;
+	ftom_zoommul = FRACUNIT;
+    }
+
     if (scale_mtof < min_scale_mtof)
 	AM_minOutWindowScale();
     else if (scale_mtof > max_scale_mtof)
@@ -1122,7 +1165,11 @@ AM_drawFline
 	return;
     }
 
+#ifndef CRISPY_TRUECOLOR
 #define PUTDOT(xx,yy,cc) fb[(yy)*f_w+(xx)]=(cc)
+#else
+#define PUTDOT(xx,yy,cc) fb[(yy)*f_w+(xx)]=(colormaps[(cc)])
+#endif
 
     dx = fl->b.x - fl->a.x;
     ax = 2 * (dx<0 ? -dx : dx);
@@ -1647,6 +1694,10 @@ AM_drawThings
 		 t->radius, t->angle,
 		// [crispy] show countable kills in red ...
 		 ((t->flags & (MF_COUNTKILL | MF_CORPSE)) == MF_COUNTKILL) ? REDS :
+		// [crispy] ... show Lost Souls and missiles in orange ...
+                (t->flags & (MF_FLOAT | MF_MISSILE)) ? 216 :
+		// [crispy] ... show other shootable items in dark gold ...
+		 (t->flags & MF_SHOOTABLE) ? 164 :
 		// [crispy] ... corpses in gray ...
 		 (t->flags & MF_CORPSE) ? GRAYS :
 		// [crispy] ... and countable items in yellow
@@ -1716,7 +1767,11 @@ void AM_drawCrosshair(int color)
 // [crispy] do not draw the useless dot on the player arrow
 /*
     else
+#ifndef CRISPY_TRUECOLOR
     fb[(f_w*(f_h+1))/2] = color; // single point for now
+#else
+    fb[(f_w*(f_h+1))/2] = colormaps[color]; // single point for now
+#endif
 */
 
 }
