@@ -64,8 +64,8 @@ typedef struct
 {
     GameMission_t mission;
     int episode, level;
-    char *background;
-    char *text;
+    const char *background;
+    const char *text;
 } textscreen_t;
 
 static textscreen_t textscreens[] =
@@ -74,6 +74,7 @@ static textscreen_t textscreens[] =
     { doom,      2, 8,  "SFLR6_1",   E2TEXT},
     { doom,      3, 8,  "MFLR8_4",   E3TEXT},
     { doom,      4, 8,  "MFLR8_3",   E4TEXT},
+    { doom,      5, 8,  "FLOOR7_2",  E5TEXT}, // [crispy] Sigil
 
     { doom2,     1, 6,  "SLIME16",   C1TEXT},
     { doom2,     1, 11, "RROCK14",   C2TEXT},
@@ -298,6 +299,7 @@ void F_TextWrite (void)
 	
     for (y=0 ; y<SCREENHEIGHT ; y++)
     {
+#ifndef CRISPY_TRUECOLOR
 	for (x=0 ; x<SCREENWIDTH/64 ; x++)
 	{
 	    memcpy (dest, src+((y&63)<<6), 64);
@@ -308,6 +310,12 @@ void F_TextWrite (void)
 	    memcpy (dest, src+((y&63)<<6), SCREENWIDTH&63);
 	    dest += (SCREENWIDTH&63);
 	}
+#else
+	for (x=0 ; x<SCREENWIDTH ; x++)
+	{
+		*dest++ = colormaps[src[((y&63)<<6) + (x&63)]];
+	}
+#endif
     }
 
     V_MarkRect (0, 0, SCREENWIDTH, SCREENHEIGHT);
@@ -368,7 +376,7 @@ void F_TextWrite (void)
 //
 typedef struct
 {
-    char		*name;
+    const char	*name;
     mobjtype_t	type;
 } castinfo_t;
 
@@ -403,7 +411,128 @@ int		castonmelee;
 boolean		castattacking;
 static signed char	castangle; // [crispy] turnable cast
 static signed char	castskip; // [crispy] skippable cast
+static boolean	castflip; // [crispy] flippable death sequence
 
+// [crispy] randomize seestate and deathstate sounds in the cast
+static int F_RandomizeSound (int sound)
+{
+	if (!crispy->soundfix)
+		return sound;
+
+	switch (sound)
+	{
+		// [crispy] actor->info->seesound, from p_enemy.c:A_Look()
+		case sfx_posit1:
+		case sfx_posit2:
+		case sfx_posit3:
+			return sfx_posit1 + Crispy_Random()%3;
+			break;
+
+		case sfx_bgsit1:
+		case sfx_bgsit2:
+			return sfx_bgsit1 + Crispy_Random()%2;
+			break;
+
+		// [crispy] actor->info->deathsound, from p_enemy.c:A_Scream()
+		case sfx_podth1:
+		case sfx_podth2:
+		case sfx_podth3:
+			return sfx_podth1 + Crispy_Random()%3;
+			break;
+
+		case sfx_bgdth1:
+		case sfx_bgdth2:
+			return sfx_bgdth1 + Crispy_Random()%2;
+			break;
+
+		default:
+			return sound;
+			break;
+	}
+}
+
+extern void A_BruisAttack();
+extern void A_BspiAttack();
+extern void A_CPosAttack();
+extern void A_CPosRefire();
+extern void A_CyberAttack();
+extern void A_FatAttack1();
+extern void A_FatAttack2();
+extern void A_FatAttack3();
+extern void A_HeadAttack();
+extern void A_PainAttack();
+extern void A_PosAttack();
+extern void A_SargAttack();
+extern void A_SkelFist();
+extern void A_SkelMissile();
+extern void A_SkelWhoosh();
+extern void A_SkullAttack();
+extern void A_SPosAttack();
+extern void A_TroopAttack();
+extern void A_VileTarget();
+
+typedef struct
+{
+	void *const action;
+	const int sound;
+	const boolean early;
+} actionsound_t;
+
+static const actionsound_t actionsounds[] =
+{
+	{A_PosAttack,   sfx_pistol, false},
+	{A_SPosAttack,  sfx_shotgn, false},
+	{A_CPosAttack,  sfx_shotgn, false},
+	{A_CPosRefire,  sfx_shotgn, false},
+	{A_VileTarget,  sfx_vilatk, true},
+	{A_SkelWhoosh,  sfx_skeswg, false},
+	{A_SkelFist,    sfx_skepch, false},
+	{A_SkelMissile, sfx_skeatk, true},
+	{A_FatAttack1,  sfx_firsht, false},
+	{A_FatAttack2,  sfx_firsht, false},
+	{A_FatAttack3,  sfx_firsht, false},
+	{A_HeadAttack,  sfx_firsht, true},
+	{A_BruisAttack, sfx_firsht, true},
+	{A_TroopAttack, sfx_claw,   false},
+	{A_SargAttack,  sfx_sgtatk, true},
+	{A_SkullAttack, sfx_sklatk, false},
+	{A_PainAttack,  sfx_sklatk, true},
+	{A_BspiAttack,  sfx_plasma, false},
+	{A_CyberAttack, sfx_rlaunc, false},
+};
+
+// [crispy] play attack sound based on state action function (instead of state number)
+static int F_SoundForState (int st)
+{
+	void *const castaction = (void *) caststate->action.acv;
+	void *const nextaction = (void *) (&states[caststate->nextstate])->action.acv;
+
+	// [crispy] fix Doomguy in casting sequence
+	if (castaction == NULL)
+	{
+		if (st == S_PLAY_ATK2)
+			return sfx_dshtgn;
+		else
+			return 0;
+	}
+	else
+	{
+		int i;
+
+		for (i = 0; i < arrlen(actionsounds); i++)
+		{
+			const actionsound_t *const as = &actionsounds[i];
+
+			if ((!as->early && castaction == as->action) ||
+			    (as->early && nextaction == as->action))
+			{
+				return as->sound;
+			}
+		}
+	}
+
+	return 0;
+}
 
 //
 // F_StartCast
@@ -448,16 +577,20 @@ void F_CastTicker (void)
 	if (castorder[castnum].name == NULL)
 	    castnum = 0;
 	if (mobjinfo[castorder[castnum].type].seesound)
-	    S_StartSound (NULL, mobjinfo[castorder[castnum].type].seesound);
+	    S_StartSound (NULL, F_RandomizeSound(mobjinfo[castorder[castnum].type].seesound));
 	caststate = &states[mobjinfo[castorder[castnum].type].seestate];
 	castframes = 0;
 	castangle = 0; // [crispy] turnable cast
+	castflip = false; // [crispy] flippable death sequence
     }
     else
     {
 	// just advance to next state in animation
+	// [crispy] fix Doomguy in casting sequence
+	/*
 	if (!castdeath && caststate == &states[S_PLAY_ATK1])
 	    goto stopattack;	// Oh, gross hack!
+	*/
 	// [crispy] Allow A_RandomJump() in deaths in cast sequence
 	if (caststate->action.acp1 == A_RandomJump && Crispy_Random() < caststate->misc2)
 	{
@@ -465,15 +598,24 @@ void F_CastTicker (void)
 	}
 	else
 	{
+	// [crispy] fix Doomguy in casting sequence
+	if (!castdeath && caststate == &states[S_PLAY_ATK1])
+	    st = S_PLAY_ATK2;
+	else
+	if (!castdeath && caststate == &states[S_PLAY_ATK2])
+	    goto stopattack;	// Oh, gross hack!
+	else
 	st = caststate->nextstate;
 	}
 	caststate = &states[st];
 	castframes++;
 	
+	sfx = F_SoundForState(st);
+/*
 	// sound hacks....
 	switch (st)
 	{
-	  case S_PLAY_ATK1:	sfx = sfx_dshtgn; break;
+	  case S_PLAY_ATK2:	sfx = sfx_dshtgn; break; // [crispy] fix Doomguy in casting sequence
 	  case S_POSS_ATK2:	sfx = sfx_pistol; break;
 	  case S_SPOS_ATK2:	sfx = sfx_shotgn; break;
 	  case S_VILE_ATK2:	sfx = sfx_vilatk; break;
@@ -502,6 +644,7 @@ void F_CastTicker (void)
 	  default: sfx = 0; break;
 	}
 		
+*/
 	if (sfx)
 	    S_StartSound (NULL, sfx);
     }
@@ -635,8 +778,14 @@ boolean F_CastResponder (event_t* ev)
         S_StartSound (NULL, sfx_slop);
     else
     if (mobjinfo[castorder[castnum].type].deathsound)
-	S_StartSound (NULL, mobjinfo[castorder[castnum].type].deathsound);
+	S_StartSound (NULL, F_RandomizeSound(mobjinfo[castorder[castnum].type].deathsound));
 	
+    // [crispy] flippable death sequence
+    castflip = crispy->flipcorpses &&
+	castdeath &&
+	(mobjinfo[castorder[castnum].type].flags & MF_FLIPPABLE) &&
+	(Crispy_Random() & 1);
+
     return true;
 }
 
@@ -718,7 +867,7 @@ void F_CastDrawer (void)
     }
     sprframe = &sprdef->spriteframes[ caststate->frame & FF_FRAMEMASK];
     lump = sprframe->lump[castangle]; // [crispy] turnable cast
-    flip = (boolean)sprframe->flip[castangle]; // [crispy] turnable cast
+    flip = (boolean)sprframe->flip[castangle] ^ castflip; // [crispy] turnable cast, flippable death sequence
 			
     patch = W_CacheLumpNum (lump+firstspritelump, PU_CACHE);
     if (flip)
@@ -858,6 +1007,14 @@ static void F_ArtScreenDrawer(void)
             case 4:
                 lumpname = "ENDPIC";
                 break;
+            // [crispy] Sigil
+            case 5:
+                lumpname = "SIGILEND";
+                if (W_CheckNumForName(DEH_String(lumpname)) == -1)
+                {
+                    return;
+                }
+                break;
             default:
                 return;
         }
