@@ -751,7 +751,7 @@ void G_BuildTiccmd (ticcmd_t* cmd, int maketic)
     { 
 	sendpause = false; 
 	// [crispy] ignore un-pausing in menus during demo recording
-	if (!(menuactive && demorecording && paused))
+	if (!(menuactive && demorecording && paused) && gameaction != ga_loadgame)
 	{
 	cmd->buttons = BT_SPECIAL | BTS_PAUSE; 
 	}
@@ -1656,9 +1656,12 @@ void G_DoCompleted (void)
         {
             switch(gamemap)
             {
+            // [crispy] display tally screen after ExM8
+            /*
               case 8:
                 gameaction = ga_victory;
                 return;
+            */
               case 9: 
                 for (i=0 ; i<MAXPLAYERS ; i++) 
                     players[i].didsecret = true; 
@@ -1667,6 +1670,8 @@ void G_DoCompleted (void)
         }
     }
 
+// [crispy] disable redundant code
+/*
 //#if 0  Hmmm - why?
     if ( (gamemap == 8)
 	 && (gamemode != commercial) ) 
@@ -1684,6 +1689,7 @@ void G_DoCompleted (void)
 	    players[i].didsecret = true; 
     } 
 //#endif
+*/
     
 	 
     wminfo.didsecret = players[consoleplayer].didsecret; 
@@ -1691,7 +1697,7 @@ void G_DoCompleted (void)
     wminfo.last = gamemap -1;
     
     // wminfo.next is 0 biased, unlike gamemap
-    if ( gamemission == pack_nerve && crispy->singleplayer )
+    if ( gamemission == pack_nerve && gamemap <= 9 && crispy->singleplayer )
     {
 	if (secretexit)
 	    switch(gamemap)
@@ -1706,7 +1712,7 @@ void G_DoCompleted (void)
 	    }
     }
     else
-    if ( gamemission == pack_master && crispy->singleplayer )
+    if ( gamemission == pack_master && gamemap <= 21 && crispy->singleplayer )
     {
 	wminfo.next = gamemap;
     }
@@ -1754,6 +1760,7 @@ void G_DoCompleted (void)
 		wminfo.next = 5; 
 		break; 
 	      case 3: 
+	      case 5: // [crispy] Sigil
 		wminfo.next = 6; 
 		break; 
 	      case 4:
@@ -1773,12 +1780,7 @@ void G_DoCompleted (void)
     wminfo.maxsecret = totalsecret; 
     wminfo.maxfrags = 0; 
 
-    if (gamemap == 33 || (crispy->havee1m10 && gameepisode == 1 && gamemap == 10))
-    {
-        // [crispy] par time for inofficial maps sucks
-        wminfo.partime = INT_MAX;
-    }
-    else
+    // [crispy] single player par times for NRFTL
     if (gamemission == pack_nerve && crispy->singleplayer)
     {
         wminfo.partime = TICRATE*npars[gamemap-1];
@@ -1789,7 +1791,7 @@ void G_DoCompleted (void)
     if (gamemode == commercial)
     {
         // map33 has no official time: initialize to zero
-        if (gamemap == 33 && false) // [crispy] disable
+        if (gamemap == 33)
         {
             wminfo.partime = 0;
         }
@@ -1816,10 +1818,16 @@ void G_DoCompleted (void)
         else
         wminfo.partime = TICRATE*pars[gameepisode][gamemap];
     }
+    // [crispy] single player par times for episode 4
     else if (gameepisode == 4 && crispy->singleplayer)
     {
         wminfo.partime = TICRATE*e4pars[gamemap];
     }
+    // [crispy] use episode 3 par times for Sigil's episode 5
+    else if (gameepisode == 5)
+    {
+        wminfo.partime = TICRATE*pars[3][gamemap];
+    }
     else
     {
         wminfo.partime = TICRATE*cpars[gamemap];
@@ -1907,6 +1915,12 @@ void G_WorldDone (void)
 	    break;
 	}
     }
+    // [crispy] display tally screen after ExM8
+    else
+    if ( gamemap == 8 )
+    {
+	gameaction = ga_victory;
+    }
 } 
  
 void G_DoWorldDone (void) 
@@ -2346,12 +2360,13 @@ G_InitNew
 
     if (gamemode == commercial)
     {
-        if (gamemap < 12)
-            skytexturename = "SKY1";
-        else if (gamemap < 21)
-            skytexturename = "SKY2";
-        else
-            skytexturename = "SKY3";
+        skytexturename = DEH_String("SKY3");
+        skytexture = R_TextureNumForName(skytexturename);
+        if (gamemap < 21)
+        {
+            skytexturename = DEH_String(gamemap < 12 ? "SKY1" : "SKY2");
+            skytexture = R_TextureNumForName(skytexturename);
+        }
     }
     else
     {
@@ -2370,14 +2385,18 @@ G_InitNew
           case 4:        // Special Edition sky
             skytexturename = "SKY4";
             break;
+          case 5:        // [crispy] Sigil
+            skytexturename = "SKY5_ZD";
+            if (R_CheckTextureNumForName(DEH_String(skytexturename)) == -1)
+            {
+                skytexturename = "SKY3";
+            }
+            break;
         }
+        skytexturename = DEH_String(skytexturename);
+        skytexture = R_TextureNumForName(skytexturename);
     }
 
-    skytexturename = DEH_String(skytexturename);
-
-    skytexture = R_TextureNumForName(skytexturename);
-
-
     G_DoLoadLevel ();
 }
 
@@ -2389,6 +2408,8 @@ G_InitNew
 
 // [crispy] demo progress bar and timer widget
 int defdemotics = 0, deftotaldemotics;
+// [crispy] moved here
+static const char *defdemoname;
 
 void G_ReadDemoTiccmd (ticcmd_t* cmd) 
 { 
@@ -2398,6 +2419,30 @@ void G_ReadDemoTiccmd (ticcmd_t* cmd)
 	G_CheckDemoStatus (); 
 	return; 
     } 
+
+    // [crispy] if demo playback is quit by pressing 'q',
+    // stay in the game, hand over controls to the player and
+    // continue recording the demo under a different name
+    if (gamekeydown[key_demo_quit] && singledemo && !netgame)
+    {
+	byte *actualbuffer = demobuffer;
+	char *actualname = M_StringDuplicate(defdemoname);
+
+	gamekeydown[key_demo_quit] = false;
+
+	// [crispy] find a new name for the continued demo
+	G_RecordDemo(actualname);
+	free(actualname);
+
+	// [crispy] discard the newly allocated demo buffer
+	Z_Free(demobuffer);
+	demobuffer = actualbuffer;
+
+	// [crispy] continue recording
+	G_CheckDemoStatus();
+	return;
+    }
+
     cmd->forwardmove = ((signed char)*demo_p++); 
     cmd->sidemove = ((signed char)*demo_p++); 
 
@@ -2530,14 +2575,14 @@ void G_RecordDemo (char *name)
     FILE *fp = NULL;
 
     usergame = false;
-    demoname_size = strlen(name) + 5 + 4; // [crispy] + 4 for "-000"
+    demoname_size = strlen(name) + 5 + 6; // [crispy] + 6 for "-00000"
     demoname = Z_Malloc(demoname_size, PU_STATIC, NULL);
     M_snprintf(demoname, demoname_size, "%s.lmp", name);
 
     // [crispy] prevent overriding demos by adding a file name suffix
-    for (i = 0; i <= 999 && (fp = fopen(demoname, "rb")) != NULL; i++)
+    for (i = 0; i <= 99999 && (fp = fopen(demoname, "rb")) != NULL; i++)
     {
-	M_snprintf(demoname, demoname_size, "%s-%03d.lmp", name, i);
+	M_snprintf(demoname, demoname_size, "%s-%05d.lmp", name, i);
 	fclose (fp);
     }
 
@@ -2624,7 +2669,6 @@ void G_BeginRecording (void)
 // G_PlayDemo 
 //
 
-static const char *defdemoname;
  
 void G_DeferedPlayDemo(const char *name)
 { 
@@ -2763,6 +2807,8 @@ void G_DoPlayDemo (void)
     {
 	netgame = true;
 	netdemo = true;
+	// [crispy] impossible to continue a multiplayer demo
+	demorecording = false;
     }
 
     // don't spend a lot of time in loadlevel 
@@ -2870,9 +2916,13 @@ boolean G_CheckDemoStatus (void)
 	netgame = false;
 	deathmatch = false;
 	playeringame[1] = playeringame[2] = playeringame[3] = 0;
+	// [crispy] leave game parameters intact when continuing a demo
+	if (!demorecording)
+	{
 	respawnparm = false;
 	fastparm = false;
 	nomonsters = false;
+	}
 	consoleplayer = 0;
         
         // [crispy] in demo continue mode increase the demo buffer and
@@ -2885,6 +2935,12 @@ boolean G_CheckDemoStatus (void)
             nodrawers = false;
             singletics = false;
 
+            // [crispy] start music for the current level
+            if (gamestate == GS_LEVEL)
+            {
+                S_Start();
+            }
+
             return true;
         }
 
