@@ -45,6 +45,7 @@
 #include "dstrings.h"
 #include "sounds.h"
 
+#include "r_state.h" // [crispy] colormaps
 #include "v_video.h" // [crispy] V_DrawPatch() et al.
 #include "v_trans.h" // [crispy] colored kills/items/secret/etc. messages
 
@@ -176,6 +177,17 @@ const char *mapnames[] =	// DOOM shareware/registered/retail (Ultimate) names.
     HUSTR_E4M8,
     HUSTR_E4M9,
 
+    // [crispy] Sigil
+    HUSTR_E5M1,
+    HUSTR_E5M2,
+    HUSTR_E5M3,
+    HUSTR_E5M4,
+    HUSTR_E5M5,
+    HUSTR_E5M6,
+    HUSTR_E5M7,
+    HUSTR_E5M8,
+    HUSTR_E5M9,
+
     "NEWLEVEL",
     "NEWLEVEL",
     "NEWLEVEL",
@@ -403,7 +415,7 @@ static void CrispyReplaceColor (char *str, const int cr, const char *col)
 {
     char *str_replace, col_replace[16];
 
-    if (strcmp(str, DEH_String(str)))
+    if (DEH_HasStringReplacement(str))
     {
 	return;
     }
@@ -709,7 +721,7 @@ void HU_Start(void)
 
     // [crispy] explicitely display (episode and) map if the
     // map is from a PWAD or if the map title string has been dehacked
-    if (strcmp(s, DEH_String(s)) || (!W_IsIWADLump(maplumpinfo) && (!nervewadfile || gamemission != pack_nerve)))
+    if (DEH_HasStringReplacement(s) || (!W_IsIWADLump(maplumpinfo) && (!nervewadfile || gamemission != pack_nerve)))
     {
 	char *m;
 
@@ -755,12 +767,21 @@ void HU_DemoProgressBar (void)
 {
     const int i = SCREENWIDTH * defdemotics / deftotaldemotics;
 
+#ifndef CRISPY_TRUECOLOR
 //  V_DrawHorizLine(0, SCREENHEIGHT - 3, i, 4); // [crispy] white
     V_DrawHorizLine(0, SCREENHEIGHT - 2, i, 0); // [crispy] black
     V_DrawHorizLine(0, SCREENHEIGHT - 1, i, 4); // [crispy] white
 
 //  V_DrawHorizLine(0, SCREENHEIGHT - 2, 1, 4); // [crispy] white start
 //  V_DrawHorizLine(i - 1, SCREENHEIGHT - 2, 1, 4); // [crispy] white end
+#else
+//  V_DrawHorizLine(0, SCREENHEIGHT - 3, i, colormaps[4]); // [crispy] white
+    V_DrawHorizLine(0, SCREENHEIGHT - 2, i, colormaps[0]); // [crispy] black
+    V_DrawHorizLine(0, SCREENHEIGHT - 1, i, colormaps[4]); // [crispy] white
+
+//  V_DrawHorizLine(0, SCREENHEIGHT - 2, 1, colormaps[4]); // [crispy] white start
+//  V_DrawHorizLine(i - 1, SCREENHEIGHT - 2, 1, colormaps[4]); // [crispy] white end
+#endif
 }
 
 // [crispy] static, non-projected crosshair
@@ -922,10 +943,13 @@ void HU_Ticker(void)
 	// [crispy] display centered message
 	if (plr->centermessage)
 	{
+	    extern int M_StringWidth(const char *string);
+	    w_secret.l[0].x = ORIGWIDTH/2 - M_StringWidth(plr->centermessage)/2;
+
 	    HUlib_addMessageToSText(&w_secret, 0, plr->centermessage);
 	    plr->centermessage = 0;
 	    secret_on = true;
-	    secret_counter = HU_MSGTIMEOUT >> 1;
+	    secret_counter = 5*TICRATE/2; // [crispy] 2.5 seconds
 	}
 
 	// display message if necessary
