@@ -236,7 +236,7 @@ static void M_DrawSaveLoadBorder(int x,int y);
 static void M_SetupNextMenu(menu_t *menudef);
 static void M_DrawThermo(int x,int y,int thermWidth,int thermDot);
 static void M_WriteText(int x, int y, const char *string);
-static int  M_StringWidth(const char *string);
+int  M_StringWidth(const char *string); // [crispy] un-static
 static int  M_StringHeight(const char *string);
 static void M_StartMessage(const char *string, void *routine, boolean input);
 static void M_ClearMenus (void);
@@ -297,6 +297,7 @@ enum
     ep2,
     ep3,
     ep4,
+    ep5, // [crispy] Sigil
     ep_end
 } episodes_e;
 
@@ -306,6 +307,7 @@ menuitem_t EpisodeMenu[]=
     {1,"M_EPI2", M_Episode,'t'},
     {1,"M_EPI3", M_Episode,'i'},
     {1,"M_EPI4", M_Episode,'t'}
+   ,{1,"M_EPI5", M_Episode,'s'} // [crispy] Sigil
 };
 
 menu_t  EpiDef =
@@ -964,16 +966,25 @@ void M_DoSave(int slot)
 static void SetDefaultSaveName(int slot)
 {
     // map from IWAD or PWAD?
-    if (W_IsIWADLump(maplumpinfo))
+    if (W_IsIWADLump(maplumpinfo) && strcmp(savegamedir, ""))
     {
         M_snprintf(savegamestrings[itemOn], SAVESTRINGSIZE,
                    "%s", maplumpinfo->name);
     }
     else
     {
+        char *wadname = M_StringDuplicate(W_WadNameForLump(maplumpinfo));
+        char *ext = strrchr(wadname, '.');
+
+        if (ext != NULL)
+        {
+            *ext = '\0';
+        }
+
         M_snprintf(savegamestrings[itemOn], SAVESTRINGSIZE,
-                   "%s: %s", W_WadNameForLump(maplumpinfo),
-                   maplumpinfo->name);
+                   "%s (%s)", maplumpinfo->name,
+                   wadname);
+        free(wadname);
     }
     M_ForceUppercase(savegamestrings[itemOn]);
     joypadSave = false;
@@ -1403,7 +1414,11 @@ static void M_DrawCrispnessBackground(void)
 	{
 		for (x = 0; x < SCREENWIDTH; x++)
 		{
+#ifndef CRISPY_TRUECOLOR
 			*dest++ = src[(y & 63) * 64 + (x & 63)];
+#else
+			*dest++ = colormaps[src[(y & 63) * 64 + (x & 63)]];
+#endif
 		}
 	}
 
@@ -1487,7 +1502,7 @@ static void M_DrawCrispness2(void)
     M_DrawCrispnessSeparator(crispness_sep_audible, "Audible");
     M_DrawCrispnessItem(crispness_soundfull, "Play sounds in full length", crispy->soundfull, true);
     M_DrawCrispnessItem(crispness_soundfix, "Misc. Sound Fixes", crispy->soundfix, true);
-    M_DrawCrispnessMultiItem(crispness_sndchannels, "Sound Channels", multiitem_sndchannels, snd_channels >> 4, true);
+    M_DrawCrispnessMultiItem(crispness_sndchannels, "Sound Channels", multiitem_sndchannels, snd_channels >> 4, snd_sfxdevice != SNDDEVICE_PCSPEAKER);
     M_DrawCrispnessItem(crispness_soundmono, "Mono SFX", crispy->soundmono, true);
 
     M_DrawCrispnessSeparator(crispness_sep_navigational, "Navigational");
@@ -1495,7 +1510,7 @@ static void M_DrawCrispness2(void)
     M_DrawCrispnessMultiItem(crispness_automapstats, "Show Level Stats", multiitem_widgets, crispy->automapstats, true);
     M_DrawCrispnessMultiItem(crispness_leveltime, "Show Level Time", multiitem_widgets, crispy->leveltime, true);
     M_DrawCrispnessMultiItem(crispness_playercoords, "Show Player Coords", multiitem_widgets, crispy->playercoords, true);
-    M_DrawCrispnessItem(crispness_secretmessage, "Show Revealed Secrets", crispy->secretmessage, true);
+    M_DrawCrispnessMultiItem(crispness_secretmessage, "Show Revealed Secrets", multiitem_secretmessage, crispy->secretmessage, true);
 
     M_DrawCrispnessGoto(crispness2_next, "Next Page >");
     M_DrawCrispnessGoto(crispness2_prev, "< Prev Page");
@@ -2076,11 +2091,12 @@ static int G_ReloadLevel(void)
 
 static int G_GotoNextLevel(void)
 {
-  static byte doom_next[4][9] = {
+  static byte doom_next[5][9] = {
     {12, 13, 19, 15, 16, 17, 18, 21, 14},
     {22, 23, 24, 25, 29, 27, 28, 31, 26},
     {32, 33, 34, 35, 36, 39, 38, 41, 37},
-    {42, 49, 44, 45, 46, 47, 48, 11, 43}
+    {42, 49, 44, 45, 46, 47, 48, 51, 43},
+    {52, 53, 54, 55, 56, 59, 58, 11, 57},
   };
   static byte doom2_next[33] = {
     0, 3, 4, 5, 6, 7, 8, 9, 10, 11,
@@ -2127,6 +2143,9 @@ static int G_GotoNextLevel(void)
       if (gamemode == registered)
         doom_next[2][7] = 11;
 
+      if (!crispy->haved1e5)
+        doom_next[3][7] = 11;
+
       if (gameversion == exe_chex)
       {
         doom_next[0][2] = 14;
@@ -2586,7 +2605,18 @@ boolean M_Responder (event_t* ev)
 	    if (usegamma > 4+4) // [crispy] intermediate gamma levels
 		usegamma = 0;
 	    players[consoleplayer].message = DEH_String(gammamsg[usegamma]);
+#ifndef CRISPY_TRUECOLOR
             I_SetPalette (W_CacheLumpName (DEH_String("PLAYPAL"),PU_CACHE));
+#else
+            {
+		extern void R_InitColormaps (void);
+		I_SetPalette (0);
+		R_InitColormaps();
+		inhelpscreens = true;
+		R_FillBackScreen();
+		viewactive = false;
+            }
+#endif
 	    return true;
 	}
         // [crispy] those two can be considered as shortcuts for the IDCLEV cheat
@@ -2733,6 +2763,7 @@ boolean M_Responder (event_t* ev)
     // [crispy] next/prev Crispness menu
     else if (key == KEY_PGUP)
     {
+	currentMenu->lastOn = itemOn;
 	if (currentMenu == CrispnessMenus[crispness_cur])
 	{
 	    M_CrispnessPrev(0);
@@ -2742,6 +2773,7 @@ boolean M_Responder (event_t* ev)
     }
     else if (key == KEY_PGDN)
     {
+	currentMenu->lastOn = itemOn;
 	if (currentMenu == CrispnessMenus[crispness_cur])
 	{
 	    M_CrispnessNext(0);
@@ -3044,6 +3076,12 @@ void M_Init (void)
         ReadMenu1[rdthsempty1].routine = M_FinishReadThis;
     }
 
+    // [crispy] Sigil
+    if (!crispy->haved1e5)
+    {
+        EpiDef.numitems = 4;
+    }
+
     // Versions of doom.exe before the Ultimate Doom release only had
     // three episodes; if we're emulating one of those then don't try
     // to show episode four. If we are, then do show episode four
@@ -3097,7 +3135,7 @@ void M_Init (void)
 
 	// [crispy] "i wouldn't leave if i were you.\ndos is much worse."
 	string = doom1_endmsg[3];
-	if (!strcmp(string, DEH_String(string)))
+	if (!DEH_HasStringReplacement(string))
 	{
 		replace = M_StringReplace(string, "dos", crispy->platform);
 		DEH_AddStringReplacement(string, replace);
@@ -3106,7 +3144,7 @@ void M_Init (void)
 
 	// [crispy] "you're trying to say you like dos\nbetter than me, right?"
 	string = doom1_endmsg[4];
-	if (!strcmp(string, DEH_String(string)))
+	if (!DEH_HasStringReplacement(string))
 	{
 		replace = M_StringReplace(string, "dos", crispy->platform);
 		DEH_AddStringReplacement(string, replace);
@@ -3115,7 +3153,7 @@ void M_Init (void)
 
 	// [crispy] "don't go now, there's a \ndimensional shambler waiting\nat the dos prompt!"
 	string = doom2_endmsg[2];
-	if (!strcmp(string, DEH_String(string)))
+	if (!DEH_HasStringReplacement(string))
 	{
 		replace = M_StringReplace(string, "dos", "command");
 		DEH_AddStringReplacement(string, replace);
