@@ -48,36 +48,37 @@ mapformat_t P_CheckMapFormat (int lumpnum)
     if ((b = lumpnum+ML_BLOCKMAP+1) < numlumps &&
         !strncasecmp(lumpinfo[b]->name, "BEHAVIOR", 8))
     {
-	fprintf(stderr, "Hexen map format, ");
+	fprintf(stderr, "Hexen (");
 	format |= MFMT_HEXEN;
     }
     else
-	fprintf(stderr, "Doom map format, ");
+	fprintf(stderr, "Doom (");
 
     if (!((b = lumpnum+ML_NODES) < numlumps &&
         (nodes = W_CacheLumpNum(b, PU_CACHE)) &&
         W_LumpLength(b) > 0))
-	fprintf(stderr, "no nodes.\n");
+	fprintf(stderr, "no nodes");
     else
     if (!memcmp(nodes, "xNd4\0\0\0\0", 8))
     {
-	fprintf(stderr, "DeePBSP nodes.\n");
+	fprintf(stderr, "DeePBSP");
 	format |= MFMT_DEEPBSP;
     }
     else
     if (!memcmp(nodes, "XNOD", 4))
     {
-	fprintf(stderr, "uncompressed ZDBSP nodes.\n");
+	fprintf(stderr, "ZDBSP");
 	format |= MFMT_ZDBSPX;
     }
     else
     if (!memcmp(nodes, "ZNOD", 4))
     {
-	fprintf(stderr, "compressed ZDBSP nodes.\n");
+	fprintf(stderr, "compressed ZDBSP");
 	format |= MFMT_ZDBSPZ;
     }
     else
-	fprintf(stderr, "normal BSP nodes.\n");
+	fprintf(stderr, "BSP");
+    fprintf(stderr, ")\n");
 
     if (nodes)
 	W_ReleaseLumpNum(b);
