@@ -1090,12 +1090,12 @@ static void P_LoadReject(int lumpnum)
 // [crispy] log game skill in plain text
 const char *skilltable[] =
 {
-    "No Items",
-    "ITYTD",
-    "HNTR",
-    "HMP",
-    "UV",
-    "NM"
+    "Nothing",
+    "Baby",
+    "Easy",
+    "Normal",
+    "Hard",
+    "Nightmare"
 };
 
 // [crispy] factor out map lump name and number finding into a separate function
@@ -1254,7 +1254,7 @@ P_SetupLevel
 	    nomonsters ? " -nomonsters" : "",
 	    NULL);
 
-	fprintf(stderr, "P_SetupLevel: %s (%s), %s%s, Time %d:%02d:%02d, Total %d:%02d:%02d, ",
+	fprintf(stderr, "P_SetupLevel: %s (%s) %s%s %d:%02d:%02d/%d:%02d:%02d ",
 	    maplumpinfo->name, W_WadNameForLump(maplumpinfo),
 	    skilltable[BETWEEN(0,5,(int) skill+1)], rfn_str,
 	    ltime/3600, (ltime%3600)/60, ltime%60,
