@@ -132,40 +132,56 @@ R_InstallSpriteLump
     if (rotation == 0)
     {
 	// the lump should be used for all rotations
+	// [crispy] make non-fatal
 	if (sprtemp[frame].rotate == false)
-	    I_Error ("R_InitSprites: Sprite %s frame %c has "
-		     "multip rot=0 lump", spritename, 'A'+frame);
+	    fprintf (stderr, "R_InitSprites: Sprite %s frame %c has "
+		     "multip rot=0 lump\n", spritename, 'A'+frame);
 
 	// [crispy] make non-fatal
 	if (sprtemp[frame].rotate == true)
 	    fprintf (stderr, "R_InitSprites: Sprite %s frame %c has rotations "
 		     "and a rot=0 lump\n", spritename, 'A'+frame);
 			
-	sprtemp[frame].rotate = false;
-	for (r=0 ; r<16 ; r++) // [crispy] support 16 sprite rotations
+// [crispy] moved ...
+//	sprtemp[frame].rotate = false;
+	for (r=0 ; r<8 ; r++)
 	{
+	  // [crispy] only if not yet substituted
+	  if (sprtemp[frame].lump[r] == -1)
+	  {
 	    sprtemp[frame].lump[r] = lump - firstspritelump;
 	    sprtemp[frame].flip[r] = (byte)flipped;
+	    // [crispy] ... here
+	    sprtemp[frame].rotate = false;
+	  }
 	}
 	return;
     }
 	
     // the lump is only used for one rotation
+    // [crispy] make non-fatal
     if (sprtemp[frame].rotate == false)
-	I_Error ("R_InitSprites: Sprite %s frame %c has rotations "
-		 "and a rot=0 lump", spritename, 'A'+frame);
+	fprintf (stderr, "R_InitSprites: Sprite %s frame %c has rotations "
+		 "and a rot=0 lump\n", spritename, 'A'+frame);
 		
-    sprtemp[frame].rotate = true;
+// [crispy] moved ...
+//    sprtemp[frame].rotate = true;
 
     // make 0 based
     rotation--;		
     if (sprtemp[frame].lump[rotation] != -1)
-	I_Error ("R_InitSprites: Sprite %s : %c : %c "
-		 "has two lumps mapped to it",
+    {
+	// [crispy] make non-fatal
+	fprintf (stderr, "R_InitSprites: Sprite %s : %c : %c "
+		 "has two lumps mapped to it\n",
 		 spritename, 'A'+frame, '1'+rotation);
+	return;
+    }
 		
     sprtemp[frame].lump[rotation] = lump - firstspritelump;
     sprtemp[frame].flip[rotation] = (byte)flipped;
+    // [crispy] ... here
+    sprtemp[frame].rotate = true;
 }
 
 
@@ -186,9 +202,9 @@ R_InstallSpriteLump
 //  letter/number appended.
 // The rotation character can be 0 to signify no rotations.
 //
-void R_InitSpriteDefs (char** namelist) 
+void R_InitSpriteDefs(const char **namelist)
 { 
-    char**	check;
+    const char **check;
     int		i;
     int		l;
     int		frame;
@@ -318,7 +334,7 @@ static int	numvissprites;
 // R_InitSprites
 // Called at program start.
 //
-void R_InitSprites (char** namelist)
+void R_InitSprites(const char **namelist)
 {
     int		i;
 	
@@ -494,6 +510,9 @@ R_DrawVisSprite
 	{
 	    colfunc = tlcolfunc;
 	}
+#ifdef CRISPY_TRUECOLOR
+	blendfunc = vis->blendfunc;
+#endif
     }
 	
     dc_iscale = abs(vis->xiscale)>>detailshift;
@@ -524,6 +543,9 @@ R_DrawVisSprite
     }
 
     colfunc = basecolfunc;
+#ifdef CRISPY_TRUECOLOR
+    blendfunc = I_BlendOver;
+#endif
 }
 
 
@@ -575,7 +597,7 @@ void R_ProjectSprite (mobj_t* thing)
         // that would necessitate turning it off for a tic.
         thing->interp == true &&
         // Don't interpolate during a paused state.
-        !paused && (!menuactive || demoplayback || netgame))
+        leveltime > oldleveltime)
     {
         interpx = thing->oldx + FixedMul(thing->x - thing->oldx, fractionaltic);
         interpy = thing->oldy + FixedMul(thing->y - thing->oldy, fractionaltic);
@@ -736,7 +758,7 @@ void R_ProjectSprite (mobj_t* thing)
     else
     {
 	// diminished light
-	index = xscale>>(LIGHTSCALESHIFT-detailshift+hires);
+	index = xscale>>(LIGHTSCALESHIFT-detailshift+crispy->hires);
 
 	if (index >= MAXLIGHTSCALE) 
 	    index = MAXLIGHTSCALE-1;
@@ -775,6 +797,14 @@ void R_ProjectSprite (mobj_t* thing)
 	    }
 	}
     }
+
+#ifdef CRISPY_TRUECOLOR
+    // [crispy] translucent sprites
+    if (thing->flags & MF_TRANSLUCENT)
+    {
+	vis->blendfunc = (thing->frame & FF_FULLBRIGHT) ? I_BlendAdd : I_BlendOver;
+    }
+#endif
 }
 
 extern void P_LineLaser (mobj_t* t1, angle_t angle, fixed_t distance, fixed_t slope);
@@ -865,7 +895,10 @@ static void R_DrawLSprite (void)
     vis->colormap[0] = vis->colormap[1] = fixedcolormap ? fixedcolormap : colormaps; // [crispy] always full brightness
     vis->brightmap = dc_brightmap;
     vis->translation = R_LaserspotColor();
-//  vis->mobjflags |= MF_TRANSLUCENT;
+#ifdef CRISPY_TRUECOLOR
+    vis->mobjflags |= MF_TRANSLUCENT;
+    vis->blendfunc = I_BlendAdd;
+#endif
     vis->xiscale = FixedDiv (FRACUNIT, xscale);
     vis->texturemid = laserspot->z - viewz;
     vis->scale = xscale<<detailshift;
@@ -980,7 +1013,7 @@ void R_DrawPSprite (pspdef_t* psp, psprnum_t psprnum) // [crispy] differentiate
     sprframe = &sprdef->spriteframes[ psp->state->frame & FF_FRAMEMASK ];
 
     lump = sprframe->lump[0];
-    flip = (boolean)sprframe->flip[0];
+    flip = (boolean)sprframe->flip[0] ^ crispy->flipweapons;
     
     // [crispy] smoothen Chainsaw idle animation
     if (state == S_SAW || state == S_SAWB)
@@ -1011,7 +1044,8 @@ void R_DrawPSprite (pspdef_t* psp, psprnum_t psprnum) // [crispy] differentiate
     // calculate edges of the shape
     tx = psp_sx-(ORIGWIDTH/2)*FRACUNIT;
 	
-    tx -= spriteoffset[lump];	
+    // [crispy] fix sprite offsets for mirrored sprites
+    tx -= flip ? 2 * tx - spriteoffset[lump] + spritewidth[lump] : spriteoffset[lump];
     x1 = (centerxfrac + FixedMul (tx,pspritescale) ) >>FRACBITS;
 
     // off the right side
@@ -1047,7 +1081,7 @@ void R_DrawPSprite (pspdef_t* psp, psprnum_t psprnum) // [crispy] differentiate
     }
     
     // [crispy] free look
-    vis->texturemid += FixedMul(((centery - viewheight / 2) << FRACBITS), vis->xiscale) >> detailshift;
+    vis->texturemid += FixedMul(((centery - viewheight / 2) << FRACBITS), pspriteiscale) >> detailshift;
 
     if (vis->x1 > x1)
 	vis->startfrac += vis->xiscale*(vis->x1-x1);
@@ -1080,7 +1114,12 @@ void R_DrawPSprite (pspdef_t* psp, psprnum_t psprnum) // [crispy] differentiate
 	
     // [crispy] translucent gun flash sprites
     if (psprnum == ps_flash)
+    {
         vis->mobjflags |= MF_TRANSLUCENT;
+#ifdef CRISPY_TRUECOLOR
+        vis->blendfunc = I_BlendOver; // I_BlendAdd;
+#endif
+    }
 
     R_DrawVisSprite (vis, vis->x1, vis->x2);
 }
@@ -1225,8 +1264,8 @@ void R_SortVisSprites (void)
 void R_DrawSprite (vissprite_t* spr)
 {
     drawseg_t*		ds;
-    int		clipbot[SCREENWIDTH]; // [crispy] 32-bit integer math
-    int		cliptop[SCREENWIDTH]; // [crispy] 32-bit integer math
+    int		clipbot[MAXWIDTH]; // [crispy] 32-bit integer math
+    int		cliptop[MAXWIDTH]; // [crispy] 32-bit integer math
     int			x;
     int			r1;
     int			r2;
