@@ -111,8 +111,13 @@ typedef enum
 #endif
 
 typedef uint8_t byte;
+#ifndef CRISPY_TRUECOLOR
 typedef uint8_t pixel_t;
 typedef int16_t dpixel_t;
+#else
+typedef uint32_t pixel_t;
+typedef int64_t dpixel_t;
+#endif
 
 #include <limits.h>
 
