@@ -222,12 +222,6 @@ void I_InitSound(boolean use_sfx_prefix)
 
     nomusic = M_CheckParm("-nomusic") > 0;
 
-// [crispy] forcefully set the SDL audio backend on Windows to directsound,
-// away from the buggy WASAPI default, not overriding previously set values
-#if defined(_WIN32)
-    SDL_setenv("SDL_AUDIODRIVER", "directsound", false);
-#endif
-
     //!
     //
     // Disable substitution music packs.
