@@ -34,6 +34,9 @@
 #include "i_video.h"
 #include "m_bbox.h"
 #include "m_misc.h"
+#ifdef CRISPY_TRUECOLOR
+#include "v_trans.h"
+#endif
 #include "v_video.h"
 #include "w_wad.h"
 #include "z_zone.h"
@@ -55,6 +58,9 @@ byte *tinttable = NULL;
 byte *tranmap = NULL;
 byte *dp_translation = NULL;
 boolean dp_translucent = false;
+#ifdef CRISPY_TRUECOLOR
+extern pixel_t *colormaps;
+#endif
 
 // villsa [STRIFE] Blending table used for Strife
 byte *xlatab = NULL;
@@ -158,19 +164,35 @@ void V_SetPatchClipCallback(vpatchclipfunc_t func)
 // [crispy] four different rendering functions
 // for each possible combination of dp_translation and dp_translucent:
 // (1) normal, opaque patch
-static const inline byte drawpatchpx00 (const byte dest, const byte source)
+static const inline pixel_t drawpatchpx00 (const pixel_t dest, const pixel_t source)
+#ifndef CRISPY_TRUECOLOR
 {return source;}
+#else
+{return colormaps[source];}
+#endif
 // (2) color-translated, opaque patch
-static const inline byte drawpatchpx01 (const byte dest, const byte source)
+static const inline pixel_t drawpatchpx01 (const pixel_t dest, const pixel_t source)
+#ifndef CRISPY_TRUECOLOR
 {return dp_translation[source];}
+#else
+{return colormaps[dp_translation[source]];}
+#endif
 // (3) normal, translucent patch
-static const inline byte drawpatchpx10 (const byte dest, const byte source)
+static const inline pixel_t drawpatchpx10 (const pixel_t dest, const pixel_t source)
+#ifndef CRISPY_TRUECOLOR
 {return tranmap[(dest<<8)+source];}
+#else
+{return I_BlendOver(dest, colormaps[source]);}
+#endif
 // (4) color-translated, translucent patch
-static const inline byte drawpatchpx11 (const byte dest, const byte source)
+static const inline pixel_t drawpatchpx11 (const pixel_t dest, const pixel_t source)
+#ifndef CRISPY_TRUECOLOR
 {return tranmap[(dest<<8)+dp_translation[source]];}
+#else
+{return I_BlendOver(dest, colormaps[dp_translation[source]]);}
+#endif
 // [crispy] array of function pointers holding the different rendering functions
-typedef const byte drawpatchpx_t (const byte dest, const byte source);
+typedef const pixel_t drawpatchpx_t (const pixel_t dest, const pixel_t source);
 static drawpatchpx_t *const drawpatchpx_a[2][2] = {{drawpatchpx11, drawpatchpx10}, {drawpatchpx01, drawpatchpx00}};
 
 static fixed_t dx, dxi, dy, dyi;
@@ -183,7 +205,7 @@ static void V_DrawPatchCrispy(int x, int y, patch_t *patch, int r)
     pixel_t *desttop;
     pixel_t *dest;
     byte *source;
-    byte *desttop2, *dest2;
+    pixel_t *desttop2, *dest2;
     int w;
 
     // [crispy] four different rendering functions
@@ -270,7 +292,11 @@ static void V_DrawPatchCrispy(int x, int y, patch_t *patch, int r)
             {
                 if (r)
                 {
+#ifndef CRISPY_TRUECOLOR
                     *dest2 = tinttable[*dest2];
+#else
+                    *dest2 = I_BlendDark(*dest2, 0x80);
+#endif
                     dest2 += SCREENWIDTH;
                 }
 
@@ -426,7 +452,11 @@ void V_DrawPatchFlipped(int x, int y, patch_t *patch)
                 // [crispy] too high
                 if (top++ >= 0)
                 {
+#ifndef CRISPY_TRUECOLOR
                     *dest = source[srccol >> FRACBITS];
+#else
+                    *dest = colormaps[source[srccol >> FRACBITS]];
+#endif
                 }
                 srccol += dyi;
                 dest += SCREENWIDTH;
@@ -715,9 +745,9 @@ void V_DrawBlock(int x, int y, int width, int height, pixel_t *src)
     } 
 } 
 
-void V_DrawScaledBlock(int x, int y, int width, int height, byte *src)
+void V_DrawScaledBlock(int x, int y, int width, int height, pixel_t *src)
 {
-    byte *dest;
+    pixel_t *dest;
     int i, j;
 
 #ifdef RANGECHECK
@@ -769,7 +799,7 @@ void V_DrawHorizLine(int x, int y, int w, int c)
     int x1;
 
     // [crispy] prevent framebuffer overflows
-    if (x + w > SCREENWIDTH)
+    if (x + w > (unsigned)SCREENWIDTH)
 	w = SCREENWIDTH - x;
 
     buf = I_VideoBuffer + SCREENWIDTH * y + x;
@@ -807,7 +837,7 @@ void V_DrawBox(int x, int y, int w, int h, int c)
 // to the screen)
 //
 
-void V_CopyScaledBuffer(byte *dest, byte *src, size_t size)
+void V_CopyScaledBuffer(pixel_t *dest, pixel_t *src, size_t size)
 {
     int i, j;
 
@@ -982,6 +1012,7 @@ void WritePNGfile(char *filename, pixel_t *data,
     int i, j;
 //  int w_factor, h_factor;
     byte *rowbuf;
+
     extern void I_RenderReadPixels(byte **data, int *w, int *h, int *p);
 
 /*
@@ -1025,8 +1056,8 @@ void WritePNGfile(char *filename, pixel_t *data,
 
     png_init_io(ppng, handle);
 
-    I_RenderReadPixels(&data, &width, &height, &j);
-    rowbuf = data;
+    I_RenderReadPixels(&palette, &width, &height, &j);
+    rowbuf = palette; // [crispy] pointer abuse!
 
     png_set_IHDR(ppng, pinfo, width, height,
                  8, PNG_COLOR_TYPE_RGB_ALPHA, PNG_INTERLACE_NONE,
@@ -1086,7 +1117,7 @@ void WritePNGfile(char *filename, pixel_t *data,
         png_write_row(ppng, rowbuf);
         rowbuf += j;
     }
-    free(data);
+    free(palette);
 
     png_write_end(ppng, pinfo);
     png_destroy_write_struct(&ppng, &pinfo);
@@ -1161,71 +1192,36 @@ void V_ScreenShot(const char *format)
 
 #define MOUSE_SPEED_BOX_WIDTH  120
 #define MOUSE_SPEED_BOX_HEIGHT 9
+#define MOUSE_SPEED_BOX_X (SCREENWIDTH - MOUSE_SPEED_BOX_WIDTH - 10)
+#define MOUSE_SPEED_BOX_Y 15
 
 //
 // V_DrawMouseSpeedBox
 //
 
-// If box is only to calibrate speed, testing relative speed (as a measure
-// of game pixels to movement units) is important whether physical mouse DPI
-// is high or low. Line resolution starts at 1 pixel per 1 move-unit: if
-// line maxes out, resolution becomes 1 pixel per 2 move-units, then per
-// 3 move-units, etc.
-
-static int linelen_multiplier = 1;
-
-void V_DrawMouseSpeedBox(int speed)
+static void DrawAcceleratingBox(int speed)
 {
-    extern int usemouse;
-    int bgcolor, bordercolor, red, black, white, yellow;
-    int box_x, box_y;
+    int red, white, yellow;
     int original_speed;
     int redline_x;
     int linelen;
-    int i;
-    boolean draw_acceleration = false;
 
-    // Get palette indices for colors for widget. These depend on the
-    // palette of the game being played.
-
-    bgcolor = I_GetPaletteIndex(0x77, 0x77, 0x77);
-    bordercolor = I_GetPaletteIndex(0x55, 0x55, 0x55);
+#ifndef CRISPY_TRUECOLOR
     red = I_GetPaletteIndex(0xff, 0x00, 0x00);
-    black = I_GetPaletteIndex(0x00, 0x00, 0x00);
-    yellow = I_GetPaletteIndex(0xff, 0xff, 0x00);
     white = I_GetPaletteIndex(0xff, 0xff, 0xff);
-
-    // If the mouse is turned off, don't draw the box at all.
-    if (!usemouse)
-    {
-        return;
-    }
-
-    // If acceleration is used, draw a box that helps to calibrate the
-    // threshold point.
-    if (mouse_threshold > 0 && fabs(mouse_acceleration - 1) > 0.01)
-    {
-        draw_acceleration = true;
-    }
-
-    // Calculate box position
-
-    box_x = SCREENWIDTH - MOUSE_SPEED_BOX_WIDTH - 10;
-    box_y = 15;
-
-    V_DrawFilledBox(box_x, box_y,
-                    MOUSE_SPEED_BOX_WIDTH, MOUSE_SPEED_BOX_HEIGHT, bgcolor);
-    V_DrawBox(box_x, box_y,
-              MOUSE_SPEED_BOX_WIDTH, MOUSE_SPEED_BOX_HEIGHT, bordercolor);
+    yellow = I_GetPaletteIndex(0xff, 0xff, 0x00);
+#else
+    red = I_MapRGB(0xff, 0x00, 0x00);
+    white = I_MapRGB(0xff, 0xff, 0xff);
+    yellow = I_MapRGB(0xff, 0xff, 0x00);
+#endif
 
     // Calculate the position of the red threshold line when calibrating
     // acceleration.  This is 1/3 of the way along the box.
 
     redline_x = MOUSE_SPEED_BOX_WIDTH / 3;
 
-    // Calculate line length
-
-    if (draw_acceleration && speed >= mouse_threshold)
+    if (speed >= mouse_threshold)
     {
         // Undo acceleration and get back the original mouse speed
         original_speed = speed - mouse_threshold;
@@ -1236,50 +1232,107 @@ void V_DrawMouseSpeedBox(int speed)
     }
     else
     {
-        linelen = speed / linelen_multiplier;
+        linelen = (speed * redline_x) / mouse_threshold;
     }
 
-    // Draw horizontal "thermometer" 
-
+    // Horizontal "thermometer"
     if (linelen > MOUSE_SPEED_BOX_WIDTH - 1)
     {
         linelen = MOUSE_SPEED_BOX_WIDTH - 1;
-        if (!draw_acceleration)
-        {
-            linelen_multiplier++;
-        }
     }
 
-    V_DrawHorizLine(box_x + 1, box_y + 4, MOUSE_SPEED_BOX_WIDTH - 2, black);
-
-    if (!draw_acceleration || linelen < redline_x)
+    if (linelen < redline_x)
     {
-        V_DrawHorizLine(box_x + 1, box_y + MOUSE_SPEED_BOX_HEIGHT / 2,
+        V_DrawHorizLine(MOUSE_SPEED_BOX_X + 1,
+                        MOUSE_SPEED_BOX_Y + MOUSE_SPEED_BOX_HEIGHT / 2,
                         linelen, white);
     }
     else
     {
-        V_DrawHorizLine(box_x + 1, box_y + MOUSE_SPEED_BOX_HEIGHT / 2,
+        V_DrawHorizLine(MOUSE_SPEED_BOX_X + 1,
+                        MOUSE_SPEED_BOX_Y + MOUSE_SPEED_BOX_HEIGHT / 2,
                         redline_x, white);
-        V_DrawHorizLine(box_x + redline_x, box_y + MOUSE_SPEED_BOX_HEIGHT / 2,
+        V_DrawHorizLine(MOUSE_SPEED_BOX_X + redline_x,
+                        MOUSE_SPEED_BOX_Y + MOUSE_SPEED_BOX_HEIGHT / 2,
                         linelen - redline_x, yellow);
     }
 
-    if (draw_acceleration)
+    // Draw acceleration threshold line
+    V_DrawVertLine(MOUSE_SPEED_BOX_X + redline_x, MOUSE_SPEED_BOX_Y + 1,
+                   MOUSE_SPEED_BOX_HEIGHT - 2, red);
+}
+
+// Highest seen mouse turn speed. We scale the range of the thermometer
+// according to this value, so that it never exceeds the range. Initially
+// this is set to a 1:1 setting where 1 pixel = 1 unit of speed.
+static int max_seen_speed = MOUSE_SPEED_BOX_WIDTH - 1;
+
+static void DrawNonAcceleratingBox(int speed)
+{
+    int white;
+    int linelen;
+
+#ifndef CRISPY_TRUECOLOR
+    white = I_GetPaletteIndex(0xff, 0xff, 0xff);
+#else
+    white = I_MapRGB(0xff, 0xff, 0xff);
+#endif
+
+    if (speed > max_seen_speed)
     {
-        // Draw acceleration threshold line
-        V_DrawVertLine(box_x + redline_x, box_y + 1,
-                       MOUSE_SPEED_BOX_HEIGHT - 2, red);
+        max_seen_speed = speed;
+    }
+
+    // Draw horizontal "thermometer":
+    linelen = speed * (MOUSE_SPEED_BOX_WIDTH - 1) / max_seen_speed;
+
+    V_DrawHorizLine(MOUSE_SPEED_BOX_X + 1,
+                    MOUSE_SPEED_BOX_Y + MOUSE_SPEED_BOX_HEIGHT / 2,
+                    linelen, white);
+}
+
+void V_DrawMouseSpeedBox(int speed)
+{
+    extern int usemouse;
+    int bgcolor, bordercolor, black;
+
+    // If the mouse is turned off, don't draw the box at all.
+    if (!usemouse)
+    {
+        return;
+    }
+
+    // Get palette indices for colors for widget. These depend on the
+    // palette of the game being played.
+
+#ifndef CRISPY_TRUECOLOR
+    bgcolor = I_GetPaletteIndex(0x77, 0x77, 0x77);
+    bordercolor = I_GetPaletteIndex(0x55, 0x55, 0x55);
+    black = I_GetPaletteIndex(0x00, 0x00, 0x00);
+#else
+    bgcolor = I_MapRGB(0x77, 0x77, 0x77);
+    bordercolor = I_MapRGB(0x55, 0x55, 0x55);
+    black = I_MapRGB(0x00, 0x00, 0x00);
+#endif
+
+    // Calculate box position
+
+    V_DrawFilledBox(MOUSE_SPEED_BOX_X, MOUSE_SPEED_BOX_Y,
+                    MOUSE_SPEED_BOX_WIDTH, MOUSE_SPEED_BOX_HEIGHT, bgcolor);
+    V_DrawBox(MOUSE_SPEED_BOX_X, MOUSE_SPEED_BOX_Y,
+              MOUSE_SPEED_BOX_WIDTH, MOUSE_SPEED_BOX_HEIGHT, bordercolor);
+    V_DrawHorizLine(MOUSE_SPEED_BOX_X + 1, MOUSE_SPEED_BOX_Y + 4,
+                    MOUSE_SPEED_BOX_WIDTH - 2, black);
+
+    // If acceleration is used, draw a box that helps to calibrate the
+    // threshold point.
+    if (mouse_threshold > 0 && fabs(mouse_acceleration - 1) > 0.01)
+    {
+        DrawAcceleratingBox(speed);
     }
     else
     {
-        // Draw multiplier lines to indicate current resolution
-        for (i = 1; i < linelen_multiplier; i++)
-        {
-            V_DrawVertLine(
-                box_x + (i * MOUSE_SPEED_BOX_WIDTH / linelen_multiplier),
-                box_y + 1, MOUSE_SPEED_BOX_HEIGHT - 2, yellow);
-        }
+        DrawNonAcceleratingBox(speed);
     }
 }
 
